from uuid import uuid4
from unittest.mock import MagicMock, patch

import pytest
from _pytest.monkeypatch import MonkeyPatch

from starpoint import _utils


def test__build_header_no_additional_header():
    test_uuid = uuid4()

    expected_header = {_utils.API_HEADER_KEY: str(test_uuid)}

    actual_header = _utils._build_header(api_key=test_uuid)

    assert actual_header == expected_header


def test__build_header_with_additional_header():
    test_uuid = uuid4()
    additional_header = {"test": "header"}

    expected_header = {_utils.API_HEADER_KEY: str(test_uuid)}
    expected_header.update(additional_header)

    actual_header = _utils._build_header(
        api_key=test_uuid, additional_headers=additional_header
    )

    assert actual_header == expected_header


@patch("starpoint._utils.requests")
def test__check_host_health_healty_host(requests_mock: MagicMock):
    mock_hostname = "mock_hostname"

    _utils._check_host_health(mock_hostname)

    requests_mock.get.assert_called_once_with(mock_hostname)


@patch("starpoint._utils.requests")
def test__check_host_health_not_200_host_assert(requests_mock: MagicMock):
    requests_mock.get().ok = False
    mock_hostname = "mock_hostname"

    with pytest.raises(AssertionError):
        _utils._check_host_health(mock_hostname)


@patch("starpoint._utils.requests")
def test__check_host_health_unhealthy_host_assert(
    requests_mock: MagicMock, monkeypatch: MonkeyPatch
):
    requests_mock.get().text = "unhealthy message"
    mock_hostname = "mock_hostname"

    logger_mock = MagicMock()
    monkeypatch.setattr(_utils, "LOGGER", logger_mock)

    _utils._check_host_health(mock_hostname)

    logger_mock.warning.assert_called_once()


def test__validate_host_no_host():
    with pytest.raises(ValueError, match=_utils.NO_HOST_ERROR):
        _utils._validate_host(host="")


@pytest.mark.parametrize(
    "test_host", ("asdf", "pdf://www.example.com", "www.example.com")
)
@patch("starpoint._utils._check_host_health")
def test__validate_host_invalid_url(host_health_mock: MagicMock, test_host: str):
    with pytest.raises(ValueError, match=r".*not a valid url format"):
        _utils._validate_host(host=test_host)
    host_health_mock.assert_not_called()


@pytest.mark.parametrize(
    "test_host", ("http://www.example.com/", "http://www.example.com//")
)
@patch("starpoint._utils._check_host_health")
def test__validate_host_dangling_backslash_trimmed(
    host_health_mock: MagicMock, test_host: str
):
    expected_hostname = "http://www.example.com"

    actual_hostname = _utils._validate_host(host=test_host)

    assert actual_hostname == expected_hostname
    host_health_mock.assert_called_once_with(expected_hostname)


@pytest.mark.parametrize(
    "test_host", ("http://www.example.com", "https://www.example.com")
)
@patch("starpoint._utils._check_host_health")
def test__validate_host_simple_valid_url(host_health_mock: MagicMock, test_host: str):
    expected_hostname = test_host

    actual_hostname = _utils._validate_host(host=test_host)

    assert actual_hostname == expected_hostname
    host_health_mock.assert_called_once_with(expected_hostname)


def test__check_collection_identifier_collision_neither_value():
    with pytest.raises(ValueError, match=_utils.NO_COLLECTION_VALUE_ERROR):
        _utils._check_collection_identifier_collision()


def test__check_collection_identifier_collision_both_value():
    with pytest.raises(ValueError, match=_utils.MULTI_COLLECTION_VALUE_ERROR):
        _utils._check_collection_identifier_collision(
            collection_id=uuid4(), collection_name="test_collection_name"
        )



from uuid import UUID, uuid4
from unittest.mock import MagicMock, patch

import pytest
from _pytest.monkeypatch import MonkeyPatch
from requests.exceptions import SSLError

from starpoint import reader


@pytest.fixture(scope="session")
def api_uuid() -> UUID:
    return uuid4()


@pytest.fixture(scope="session")
@patch("starpoint._utils._check_host_health")
def mock_reader(host_health_mock: MagicMock, api_uuid: UUID) -> reader.Reader:
    return reader.Reader(api_uuid)


def test_reader_default_init(mock_reader: reader.Reader, api_uuid: UUID):
    assert mock_reader.host
    assert mock_reader.host == reader.READER_URL
    assert mock_reader.api_key == api_uuid


@patch("starpoint.reader._validate_host")
def test_reader_init_non_default_host(mock_host_validator: MagicMock, api_uuid: UUID):
    test_host = "http://www.example.com"
    test_reader = reader.Reader(api_key=api_uuid, host=test_host)

    mock_host_validator.assert_called_once_with(test_host)
    # This assert needs to be after assert_called_once_with to make sure it doesn't confound the result
    assert test_reader.host == mock_host_validator()
    assert test_reader.api_key == api_uuid


@patch("starpoint.reader._check_collection_identifier_collision")
@patch("starpoint.reader.requests")
def test_reader_query_by_collection_id(
    requests_mock: MagicMock, collision_mock: MagicMock, mock_reader: reader.Reader
):
    test_uuid = uuid4()

    mock_reader.query(collection_id=test_uuid)

    collision_mock.assert_called_once_with(test_uuid, None)
    requests_mock.post.assert_called_once()


@patch("starpoint.reader._check_collection_identifier_collision")
@patch("starpoint.reader.requests")
def test_reader_query_by_collection_name(
    requests_mock: MagicMock, collision_mock: MagicMock, mock_reader: reader.Reader
):
    test_collection_name = "mock_collection_name"

    mock_reader.query(collection_name=test_collection_name)

    collision_mock.assert_called_once_with(None, test_collection_name)
    requests_mock.post.assert_called_once()


@patch("starpoint.reader.requests")
def test_reader_query_not_200(
    requests_mock: MagicMock, mock_reader: reader.Reader, monkeypatch: MonkeyPatch
):
    requests_mock.post().ok = False

    expected_json = {}

    logger_mock = MagicMock()
    monkeypatch.setattr(reader, "LOGGER", logger_mock)

    actual_json = mock_reader.query(collection_name="mock_collection_name")

    requests_mock.post.assert_called()
    logger_mock.error.assert_called_once()
    assert actual_json == expected_json


@patch("starpoint.reader.requests")
def test_reader_infer_schema_200(
    requests_mock: MagicMock, mock_reader: reader.Reader, monkeypatch: MonkeyPatch
):
    actual_json = mock_reader.infer_schema(collection_name="mock_collection_name")

    requests_mock.post.assert_called()
    assert actual_json == requests_mock.post().json()


@patch("starpoint.reader.requests")
def test_reader_infer_schema_not_200(
    requests_mock: MagicMock, mock_reader: reader.Reader, monkeypatch: MonkeyPatch
):
    requests_mock.post().ok = False

    expected_json = {}

    logger_mock = MagicMock()
    monkeypatch.setattr(reader, "LOGGER", logger_mock)

    actual_json = mock_reader.infer_schema(collection_name="mock_collection_name")

    requests_mock.post.assert_called()
    logger_mock.error.assert_called_once()
    assert actual_json == expected_json


@patch("starpoint.reader.requests")
def test_reader_query_SSLError(
    requests_mock: MagicMock, mock_reader: reader.Reader, monkeypatch: MonkeyPatch
):
    requests_mock.exceptions.SSLError = SSLError
    requests_mock.post.side_effect = SSLError("mock exception")

    logger_mock = MagicMock()
    monkeypatch.setattr(reader, "LOGGER", logger_mock)

    with pytest.raises(SSLError, match="mock exception"):
        mock_reader.query(collection_name="mock_collection_name")

    logger_mock.error.assert_called_once_with(reader.SSL_ERROR_MSG)



import logging
from pathlib import Path
from typing import Any, Dict, Iterable, List, Optional, Union

import openai

from starpoint import db, _utils

LOGGER = logging.getLogger(__name__)


NO_API_KEY_VALUE_ERROR = "Please provide at least one value for either api_key or filepath where the api key lives."
MULTI_API_KEY_VALUE_ERROR = "Please only provide either api_key or filepath with the api_key in your initialization."
NO_API_KEY_FILE_ERROR = "The provided filepath for the API key is not a valid file."

# OpenAI response error
NO_EMBEDDING_DATA_FOUND = (
    "No embedding data found in the embedding response from OpenAI."
)

DEFAULT_OPENAI_MODEL = "text-embedding-ada-002"


class OpenAIClient(object):
    def __init__(
        self,
        starpoint: db.Client,
        openai_key: Optional[str] = None,
        openai_key_filepath: Optional[str] = None,
    ):
        self.starpoint = starpoint
        self._init_openai(openai_key, openai_key_filepath)

    def _init_openai(
        self,
        openai_key: Optional[str] = None,
        openai_key_filepath: Optional[str] = None,
    ):
        """Initializes OpenAI functionality by setting up the openai client.

        Args:
            openai_key: OpenAI API key.
            openai_key_filepath: Filepath that contains an OpenAI API key.

        Raises:
            ValueError: Both API key and filepath to API key is provided. Only one is allowed.
            ValueError: No API key or filepath provided.
            ValueError: API key filepath provided is not a file.
        """
        self.openai = openai
        # TODO: maybe do this for starpoint api_key also

        if openai_key and openai_key_filepath:
            raise ValueError(MULTI_API_KEY_VALUE_ERROR)
        elif openai_key is None:
            if openai_key_filepath is None:
                raise ValueError(NO_API_KEY_VALUE_ERROR)
            if not Path(openai_key_filepath).is_file():
                raise ValueError(NO_API_KEY_FILE_ERROR)
            self.openai.api_key_path = openai_key_filepath
        else:
            self.openai.api_key = openai_key

    def build_and_insert_embeddings(
        self,
        model: str,
        input_data: Union[str, Iterable[Any]],
        document_metadatas: Optional[List[Dict[Any, Any]]] = None,
        collection_id: Optional[str] = None,
        collection_name: Optional[str] = None,
        openai_user: Optional[str] = None,
    ) -> Dict[Any, Any]:
        """Builds and inserts embeddings into starpoint by requesting embedding creation from
        an initialized openai client. Regardless whether the operation insert data into starpoint is
        successful, the response from the openai client is returned.

        Args:
            model: Embedding model to be used for creating the embeddings. For an up to date list of what
                models are available see [OpenAI's guide on embeddings](https://platform.openai.com/docs/guides/embeddings/what-are-embeddings)
            input_data: Data to be embedded.
            document_metadatas: Optional metadatas to tie to the embeddings generated using the input data.
            collection_id: The collection's id where the documents will be updated.
                This or the `collection_name` needs to be provided.
            collection_name: The collection's name where the documents will be updated.
                This or the `collection_id` needs to be provided.
            openai_user: Optional user string used by the embedding endpoint from OpenAI.

        Returns:
            dict: Responses from starpoint and openai.
        """
        _utils._check_collection_identifier_collision(collection_id, collection_name)

        embedding_response = self.openai.Embedding.create(
            model=model, input=input_data, user=openai_user
        )

        embedding_data = embedding_response.get("data")
        if embedding_data is None:
            LOGGER.warning(NO_EMBEDDING_DATA_FOUND)
            return {
                "openai_response": embedding_response,
                "starpoint_response": None,
            }

        if document_metadatas is None:
            LOGGER.info(
                "No custom document_metadatas provided. Using input_data for the document_metadatas"
            )
            if isinstance(input_data, str):
                document_metadatas = [{"input": input_data}]
            else:
                document_metadatas = [{"input": data} for data in input_data]

        # Return the embedding response no matter what issues/bugs we might run into in the sdk
        try:
            sorted_embedding_data = sorted(embedding_data, key=lambda x: x["index"])
            embeddings = map(lambda x: x.get("embedding"), sorted_embedding_data)
            starpoint_response = self.starpoint.column_insert(
                embeddings=embeddings,
                document_metadatas=document_metadatas,
                collection_id=collection_id,
                collection_name=collection_name,
            )
        except Exception as e:
            LOGGER.error(
                "An exception has occurred while trying to load embeddings into the db. "
                f"This is the error:\n{e}"
            )
            starpoint_response = {"error": str(e)}

        return {
            "openai_response": embedding_response,
            "starpoint_response": starpoint_response,
        }

    def default_build_and_insert_embeddings(
        self,
        input_data: Union[str, Iterable[Any]],
        collection_id: Optional[str] = None,
        collection_name: Optional[str] = None,
    ) -> Dict[Any, Any]:
        """Builds and inserts embeddings into starpoint by requesting embedding creation from
        an initialized openai client using the model `text-embedding-ada-002`. If you wish to use other models use the
        `build_and_insert_embeddings` method instead. Regardless whether the operation into starpoint is successful,
        the response from the openai client is returned.

        Args:
            model: Embedding model to be used for creating the embeddings. For an up to date list of what
                models are available see [OpenAI's guide on embeddings](https://platform.openai.com/docs/guides/embeddings/what-are-embeddings)
            input_data: Data to be embedded.
            document_metadatas: Optional metadatas to tie to the embeddings generated using the input data.
            collection_id: The collection's id where the documents will be updated.
                This or the `collection_name` needs to be provided.
            collection_name: The collection's name where the documents will be updated.
                This or the `collection_id` needs to be provided.
            openai_user: Optional user string used by the embedding endpoint from OpenAI.

        Returns:
            dict: Responses from starpoint and openai.
        """
        return self.build_and_insert_embeddings(
            model=DEFAULT_OPENAI_MODEL,
            input_data=input_data,
            collection_id=collection_id,
            collection_name=collection_name,
        )



from unittest.mock import MagicMock, patch
from uuid import UUID, uuid4

import pytest
from _pytest.monkeypatch import MonkeyPatch
from requests.exceptions import SSLError

from starpoint import embedding


@pytest.fixture(scope="session")
def api_uuid() -> UUID:
    return uuid4()


@pytest.fixture(scope="session")
@patch("starpoint._utils._check_host_health")
def mock_embedding_client(
    host_health_mock: MagicMock, api_uuid: UUID
) -> embedding.EmbeddingClient:
    return embedding.EmbeddingClient(api_uuid)


def test_embedding_default_init(
    mock_embedding_client: embedding.EmbeddingClient, api_uuid: UUID
):
    assert mock_embedding_client.host
    assert mock_embedding_client.host == embedding.EMBEDDING_URL
    assert mock_embedding_client.api_key == api_uuid


@patch("starpoint.embedding._validate_host")
def test_embedding_init_non_default_host(
    mock_host_validator: MagicMock, api_uuid: UUID
):
    test_host = "http://www.example.com"
    test_embedding_client = embedding.EmbeddingClient(api_key=api_uuid, host=test_host)

    mock_host_validator.assert_called_once_with(test_host)
    # This assert needs to be after assert_called_once_with to make sure it doesn't confound the result
    assert test_embedding_client.host == mock_host_validator()
    assert test_embedding_client.api_key == api_uuid


@patch("starpoint.embedding.requests")
def test_embedding_embed_not_200(
    requests_mock: MagicMock,
    mock_embedding_client: embedding.EmbeddingClient,
    monkeypatch: MonkeyPatch,
):
    requests_mock.post().ok = False

    expected_json = {}

    logger_mock = MagicMock()
    monkeypatch.setattr(embedding, "LOGGER", logger_mock)

    actual_json = mock_embedding_client.embed(["asdf"], embedding.EmbeddingModel.MINI6)

    requests_mock.post.assert_called()
    logger_mock.error.assert_called_once()
    assert actual_json == expected_json


@patch("starpoint.embedding.requests")
def test_embedding_embed_SSLError(
    requests_mock: MagicMock,
    mock_embedding_client: embedding.EmbeddingClient,
    monkeypatch: MonkeyPatch,
):
    requests_mock.exceptions.SSLError = SSLError
    requests_mock.post.side_effect = SSLError("mock exception")

    logger_mock = MagicMock()
    monkeypatch.setattr(embedding, "LOGGER", logger_mock)

    with pytest.raises(SSLError, match="mock exception"):
        mock_embedding_client.embed(["asdf"], embedding.EmbeddingModel.MINI6)

    logger_mock.error.assert_called_once_with(embedding.SSL_ERROR_MSG)



import logging
from enum import Enum
from typing import Any, Dict, List, Optional
from uuid import UUID

import requests

from starpoint._utils import (
    _build_header,
    _check_collection_identifier_collision,
    _validate_host,
)


LOGGER = logging.getLogger(__name__)

# Host
EMBEDDING_URL = "https://embedding.starpoint.ai"

# Endpoints
EMBED_PATH = "/api/v1/embed"

# Error and warning messages
SSL_ERROR_MSG = "Request failed due to SSLError. Error is likely due to invalid API key. Please check if your API is correct and still valid."


class EmbeddingModel(Enum):
    MINI6 = "MINI6"
    MINI12 = "MINI12"


class EmbeddingClient(object):
    """Client for the embedding endpoints."""

    def __init__(self, api_key: UUID, host: Optional[str] = None):
        if host is None:
            host = EMBEDDING_URL

        self.host = _validate_host(host)
        self.api_key = api_key

    def embed(
        self,
        text: List[str],
        model: EmbeddingModel,
    ) -> Dict[str, List[Dict]]:
        """Takes some text and creates an embedding against a model in starpoint.

        Args:
            text: List of strings to create embeddings from.
            model: A choice of

        Returns:
            dict: Result with multiple lists of embeddings, matching the number of requested strings to
                create embeddings from.

        Raises:
            requests.exceptions.SSLError: Failure likely due to network issues.
        """
        request_data = dict(text=text, model=model.value)
        try:
            response = requests.post(
                url=f"{self.host}{EMBED_PATH}",
                json=request_data,
                headers=_build_header(
                    api_key=self.api_key,
                    additional_headers={"Content-Type": "application/json"},
                ),
            )
        except requests.exceptions.SSLError as e:
            LOGGER.error(SSL_ERROR_MSG)
            raise

        if not response.ok:
            LOGGER.error(
                f"Request failed with status code {response.status_code} "
                f"and the following message:\n{response.text}"
            )
            return {}
        return response.json()



import logging
from typing import Any, Dict, List, Optional
from uuid import UUID

import requests

from starpoint._utils import (
    _build_header,
    _check_collection_identifier_collision,
    _validate_host,
)


LOGGER = logging.getLogger(__name__)

# Host
READER_URL = "https://reader.starpoint.ai"

# Endpoints
INFER_SCHEMA_PATH = "/api/v1/infer_schema"
QUERY_PATH = "/api/v1/query"


# Error and warning messages
SSL_ERROR_MSG = "Request failed due to SSLError. Error is likely due to invalid API key. Please check if your API is correct and still valid."


class Reader(object):
    """Client for the Reader endpoints. If you do not need to separate reading or
    writing for, consider using the [`Client` object](#client-objects).
    """

    def __init__(self, api_key: UUID, host: Optional[str] = None):
        if host is None:
            host = READER_URL

        self.host = _validate_host(host)
        self.api_key = api_key

    def query(
        self,
        sql: Optional[str] = None,
        collection_id: Optional[str] = None,
        collection_name: Optional[str] = None,
        query_embedding: Optional[List[float]] = None,
        params: Optional[List[Any]] = None,
        text_search_query: Optional[str] = None,
    ) -> Dict[Any, Any]:
        """Queries a collection. This could be by sql or query embeddings.

        Args:
            sql: Raw SQL to run against the collection.
            collection_id: The collection's id where the query will happen.
                This or the `collection_name` needs to be provided.
            collection_name: The collection's name where the query will happen.
                This or the `collection_id` needs to be provided.
            query_embedding: An embedding to query against the collection using similarity search.
            params: values for parameterized sql

        Returns:
            dict: query response json

        Raises:
            ValueError: If neither collection id and collection name are provided.
            ValueError: If both collection id and collection name are provided.
            requests.exceptions.SSLError: Failure likely due to network issues.
        """
        _check_collection_identifier_collision(collection_id, collection_name)
        # TODO: Be safe and make sure the item passed through that doesn't hold a value is a None

        # TODO: filter through query_embedding to make sure values are what we expect
        """
        dict(
            collection_id="collection_id_example",
            collection_name="collection_name_example",
            query_embedding=None,
            sql="sql_example",
        )
        """

        request_data = dict(
            collection_id=collection_id,
            collection_name=collection_name,
            query_embedding=query_embedding,
            sql=sql,
            params=params,
            text_search_query=text_search_query,
        )
        try:
            response = requests.post(
                url=f"{self.host}{QUERY_PATH}",
                json=request_data,
                headers=_build_header(
                    api_key=self.api_key,
                    additional_headers={"Content-Type": "application/json"},
                ),
            )
        except requests.exceptions.SSLError as e:
            LOGGER.error(SSL_ERROR_MSG)
            raise e

        if not response.ok:
            LOGGER.error(
                f"Request failed with status code {response.status_code} "
                f"and the following message:\n{response.text}"
            )
            return {}
        return response.json()

    def infer_schema(
        self,
        collection_id: Optional[str] = None,
        collection_name: Optional[str] = None,
    ) -> Dict[Any, Any]:
        """Infers the schema of a particular collection.
        Gives the results back by column name and the inferred type for that column.

        Args:
            collection_id: The collection's id where the query will happen.
                This or the `collection_name` needs to be provided.
            collection_name: The collection's name where the query will happen.
                This or the `collection_id` needs to be provided.

        Returns:
            dict: infer schema response json

        Raises:
            ValueError: If neither collection id and collection name are provided.
            ValueError: If both collection id and collection name are provided.
            requests.exceptions.SSLError: Failure likely due to network issues.
        """
        _check_collection_identifier_collision(collection_id, collection_name)
        # TODO: Be safe and make sure the item passed through that doesn't hold a value is a None

        """
        dict(
            collection_id="collection_id_example",
            collection_name="collection_name_example",
        )
        """

        request_data = dict(
            collection_id=collection_id,
            collection_name=collection_name,
        )
        response = requests.post(
            url=f"{self.host}{INFER_SCHEMA_PATH}",
            json=request_data,
            headers=_build_header(
                api_key=self.api_key,
                additional_headers={"Content-Type": "application/json"},
            ),
        )

        if not response.ok:
            LOGGER.error(
                f"Request failed with status code {response.status_code} "
                f"and the following message:\n{response.text}"
            )
            return {}
        return response.json()



from tempfile import NamedTemporaryFile
from uuid import uuid4
from unittest.mock import MagicMock, patch

import pytest
from _pytest.monkeypatch import MonkeyPatch

from starpoint import openai


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_init_openai_no_api_value(
    mock_writer: MagicMock, mock_reader: MagicMock
):
    with pytest.raises(ValueError, match=openai.NO_API_KEY_VALUE_ERROR):
        openai.OpenAIClient(MagicMock())


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_init_openai_both_api_value(
    mock_writer: MagicMock, mock_reader: MagicMock
):
    with pytest.raises(ValueError, match=openai.MULTI_API_KEY_VALUE_ERROR):
        openai.OpenAIClient(MagicMock(), "mock_key", "mock_path")


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_init_openai_with_api_key(
    mock_writer: MagicMock, mock_reader: MagicMock
):
    mock_api_key = "mock_key"

    client = openai.OpenAIClient(MagicMock(), openai_key=mock_api_key)

    assert client.openai.api_key == mock_api_key


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_init_openai_with_api_path(
    mock_writer: MagicMock, mock_reader: MagicMock
):
    temp_file = NamedTemporaryFile()

    client = openai.OpenAIClient(MagicMock(), openai_key_filepath=temp_file.name)

    assert client.openai.api_key_path == temp_file.name


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_init_openai_with_bad_api_path(
    mock_writer: MagicMock, mock_reader: MagicMock
):
    mock_api_key_path = "1234~/path"

    with pytest.raises(ValueError, match=openai.NO_API_KEY_FILE_ERROR):
        openai.OpenAIClient(MagicMock(), openai_key_filepath=mock_api_key_path)


@patch("starpoint.openai._utils._check_collection_identifier_collision")
@patch("starpoint.openai.OpenAIClient._init_openai")
@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_build_and_insert_embeddings_input_string_success(
    mock_writer: MagicMock,
    mock_reader: MagicMock,
    mock_init_openai: MagicMock,
    collision_mock: MagicMock,
):
    starpoint_mock = MagicMock()
    client = openai.OpenAIClient(starpoint_mock)
    openai_mock = MagicMock()
    client.openai = openai_mock

    mock_embedding = 0.77
    expected_embedding_response = {
        "data": [
            {
                "embedding": mock_embedding,
                "index": 0,
            }
        ]
    }
    openai_mock.Embedding.create.return_value = expected_embedding_response

    mock_input = "mock_input"
    mock_model = "mock-model"

    expected_build_and_insert_response = {
        "openai_response": expected_embedding_response,
        "starpoint_response": starpoint_mock.column_insert(),
    }

    actual_build_and_insert_response = client.build_and_insert_embeddings(
        model=mock_model, input_data=mock_input
    )

    assert actual_build_and_insert_response == expected_build_and_insert_response
    collision_mock.assert_called_once()

    # independently check args since embeddings is a map() generator and cannot be checked against simple equality
    insert_call_kwargs = starpoint_mock.column_insert.call_args.kwargs
    assert [mock_embedding] == list(insert_call_kwargs.get("embeddings"))
    assert [{"input": mock_input}] == insert_call_kwargs.get("document_metadatas")
    assert insert_call_kwargs.get("collection_id") is None  # default value
    assert insert_call_kwargs.get("collection_name") is None  # default value


@patch("starpoint.openai._utils._check_collection_identifier_collision")
@patch("starpoint.openai.OpenAIClient._init_openai")
@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_build_and_insert_embeddings_input_list_success(
    mock_writer: MagicMock,
    mock_reader: MagicMock,
    mock_init_openai: MagicMock,
    collision_mock: MagicMock,
):
    starpoint_mock = MagicMock()
    client = openai.OpenAIClient(starpoint_mock)
    openai_mock = MagicMock()
    client.openai = openai_mock

    expected_embedding_response = {
        "data": [
            {
                "embedding": 0.77,
                "index": 0,
            },
            {
                "embedding": 0.88,
                "index": 1,
            },
        ]
    }
    openai_mock.Embedding.create.return_value = expected_embedding_response

    mock_input = ["mock_input1", "mock_input2"]
    mock_model = "mock-model"

    expected_build_and_insert_response = {
        "openai_response": expected_embedding_response,
        "starpoint_response": starpoint_mock.column_insert(),
    }

    actual_build_and_insert_response = client.build_and_insert_embeddings(
        model=mock_model, input_data=mock_input
    )

    assert actual_build_and_insert_response == expected_build_and_insert_response
    collision_mock.assert_called_once()

    # independently check args since embeddings is a map() generator and cannot be checked against simple equality
    insert_call_kwargs = starpoint_mock.column_insert.call_args.kwargs
    assert [0.77, 0.88] == list(insert_call_kwargs.get("embeddings"))
    assert [
        {"input": "mock_input1"},
        {"input": "mock_input2"},
    ] == insert_call_kwargs.get("document_metadatas")
    assert insert_call_kwargs.get("collection_id") is None  # default value
    assert insert_call_kwargs.get("collection_name") is None  # default value


@patch("starpoint.openai._utils._check_collection_identifier_collision")
@patch("starpoint.openai.OpenAIClient._init_openai")
@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_build_and_insert_embeddings_no_data_in_response(
    mock_writer: MagicMock,
    mock_reader: MagicMock,
    mock_init_openai: MagicMock,
    collision_mock: MagicMock,
    monkeypatch: MonkeyPatch,
):
    starpoint_mock = MagicMock()
    client = openai.OpenAIClient(starpoint_mock)
    openai_mock = MagicMock()
    client.openai = openai_mock

    expected_embedding_response = {"mock": "return"}
    openai_mock.Embedding.create.return_value = expected_embedding_response

    logger_mock = MagicMock()
    monkeypatch.setattr(openai, "LOGGER", logger_mock)

    mock_input = "mock_input"
    mock_model = "mock-model"

    expected_build_and_insert_response = {
        "openai_response": expected_embedding_response,
        "starpoint_response": None,
    }

    actual_build_and_insert_response = client.build_and_insert_embeddings(
        model=mock_model, input_data=mock_input
    )

    assert actual_build_and_insert_response == expected_build_and_insert_response
    collision_mock.assert_called_once()
    starpoint_mock.column_insert.assert_not_called()
    logger_mock.warning.assert_called_once_with(openai.NO_EMBEDDING_DATA_FOUND)


@patch("starpoint.openai._utils._check_collection_identifier_collision")
@patch("starpoint.openai.OpenAIClient._init_openai")
@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_build_and_insert_embeddings_exception_during_write(
    mock_writer: MagicMock,
    mock_reader: MagicMock,
    mock_init_openai: MagicMock,
    collision_mock: MagicMock,
    monkeypatch: MonkeyPatch,
):
    starpoint_mock = MagicMock()
    client = openai.OpenAIClient(starpoint_mock)
    openai_mock = MagicMock()
    client.openai = openai_mock

    expected_embedding_response = {
        "data": [
            {
                "embedding": 0.77,
                "index": 0,
            }
        ]
    }
    openai_mock.Embedding.create.return_value = expected_embedding_response

    column_insert_error_message = "Test Exception"
    starpoint_mock.column_insert.side_effect = RuntimeError(column_insert_error_message)

    logger_mock = MagicMock()
    monkeypatch.setattr(openai, "LOGGER", logger_mock)

    mock_input = "mock_input"
    mock_model = "mock-model"

    expected_build_and_insert_response = {
        "openai_response": expected_embedding_response,
        "starpoint_response": {"error": column_insert_error_message},
    }

    build_and_insert_response = client.build_and_insert_embeddings(
        model=mock_model, input_data=mock_input
    )

    assert build_and_insert_response == expected_build_and_insert_response
    collision_mock.assert_called_once()
    logger_mock.error.assert_called_once()


@patch("starpoint.openai.OpenAIClient.build_and_insert_embeddings")
@patch("starpoint.openai._utils._check_collection_identifier_collision")
@patch("starpoint.openai.OpenAIClient._init_openai")
@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_default_build_and_insert_embeddings(
    mock_writer: MagicMock,
    mock_reader: MagicMock,
    mock_init_openai: MagicMock,
    collision_mock: MagicMock,
    build_and_insert_embeddings_mock: MagicMock,
):
    client = openai.OpenAIClient(MagicMock())
    mock_input = "mock_input"

    client.default_build_and_insert_embeddings(input_data=mock_input)

    build_and_insert_embeddings_mock.assert_called_once_with(
        model=openai.DEFAULT_OPENAI_MODEL,
        input_data=mock_input,
        collection_id=None,
        collection_name=None,
    )



from tempfile import NamedTemporaryFile
from uuid import uuid4
from unittest.mock import MagicMock, patch

import pytest
from _pytest.monkeypatch import MonkeyPatch

from starpoint import db


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_default_init(mock_writer: MagicMock, mock_reader: MagicMock):
    test_uuid = uuid4()

    client = db.Client(api_key=test_uuid)

    mock_writer.assert_called_once_with(api_key=test_uuid, host=None)
    mock_reader.assert_called_once_with(api_key=test_uuid, host=None)


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_delete(mock_writer: MagicMock, mock_reader: MagicMock):
    client = db.Client(api_key=uuid4())

    client.delete(documents=[uuid4()])

    mock_reader.assert_called_once()  # Only called during init
    mock_writer().delete.assert_called_once()


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_column_delete(mock_writer: MagicMock, mock_reader: MagicMock):
    client = db.Client(api_key=uuid4())

    client.column_delete(embeddings=[1.1], document_metadatas={"mock": "value"})

    mock_reader.assert_called_once()  # Only called during init
    mock_writer().column_delete.assert_called_once()


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_insert(mock_writer: MagicMock, mock_reader: MagicMock):
    client = db.Client(api_key=uuid4())

    client.insert(documents=[{"mock": "value"}])

    mock_reader.assert_called_once()  # Only called during init
    mock_writer().insert.assert_called_once()


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_column_insert(mock_writer: MagicMock, mock_reader: MagicMock):
    client = db.Client(api_key=uuid4())

    client.column_insert(embeddings=[1.1], document_metadatas={"mock": "value"})

    mock_reader.assert_called_once()  # Only called during init
    mock_writer().column_insert.assert_called_once()


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_query(mock_writer: MagicMock, mock_reader: MagicMock):
    client = db.Client(api_key=uuid4())

    client.query()

    mock_writer.assert_called_once()  # Only called during init
    mock_reader().query.assert_called_once()


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_infer_schema(mock_writer: MagicMock, mock_reader: MagicMock):
    client = db.Client(api_key=uuid4())

    client.infer_schema()

    mock_writer.assert_called_once()  # Only called during init
    mock_reader().infer_schema.assert_called_once()


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_update(mock_writer: MagicMock, mock_reader: MagicMock):
    client = db.Client(api_key=uuid4())

    client.update(documents=[{"mock": "value"}])

    mock_reader.assert_called_once()  # Only called during init
    mock_writer().update.assert_called_once()


@patch("starpoint.reader.Reader")
@patch("starpoint.writer.Writer")
def test_client_column_update(mock_writer: MagicMock, mock_reader: MagicMock):
    client = db.Client(api_key=uuid4())

    client.column_update(embeddings=[1.1], document_metadatas={"mock": "value"})

    mock_reader.assert_called_once()  # Only called during init
    mock_writer().column_update.assert_called_once()
